/* :folding=explicit: */

@media (min-width: 980px) {
  body {
    padding-top: 60px;
    padding-bottom: 40px;
  }
}

/* SIDEBAR 
--------------------------------------------------*/

.sidebar-nav {
  padding: 9px 0;
}

li.nav-header {
color: #666;
position: relative;
}
.nav-header-icons {
text-align: right;
position: absolute;
right: 10px;
top: 0;
}
.nav-header-icons .btn-mini {
padding: 2px 1px 2px 2px;
margin: 0 2px;
}


/* Footer
-------------------------------------------------- */
.footer {
  margin-top: 12px;
  padding: 24px 0 24px;
  border-top: 1px solid #e5e5e5;
}
.footer ul {
  list-style-type: none;
  display: inline-block;
}

/** LOGO **/
.navbar .brand {
  background: transparent url(../images/logo_penelope.png) no-repeat left top;
  text-indent: -99999em;
  padding:0;
  margin:-10px 0 0 0;
  width: 217px;
  height: 48px;
  overflow:  hidden;
  position: relative;
  top:10px;
  left:-20px;
}
.navbar-fixed-top .navbar-inner {
  min-height: 30px;
}
/** responsive **/
@media (max-width: 980px) {
  .navbar .nav-collapse {
    left: 75px;
  }
  .navbar .nav-collapse.in {
    
  }
  .navbar .brand {
    left: -25px;
    margin-top: -15px;
  }
}


iframe #docs-chrome {
  display: none;
}


/* leave enough space for feedback */
body .modal-body {
  max-height: 600px;
}

.highlight {
  padding: 1px 3px;
  background-color: #FEE9CC;
}


/*
**  ====================
**  GENERIC FORM STYLING
**  ====================
*/


.ui-datepicker-trigger {
  /* calendar icon */
  width: 24px;
  height: 24px;
  vertical-align: middle;
}

.ui-datepicker-title {
  /* draggable */
  cursor: pointer;
}

/* Bootstrap's text areas are very small */
body textarea {
  width: auto;
}



/* ----- Inline forms ----- */

.form-inline .control-group,
.form-inline .controls {
  display: inline;
}

.form-inline input.hasDatepicker {
  width: auto;
}

.form-inline {
  /* space between lines of widgets */
  line-height: 38px;
}

.form-inline select {
  /* width of non-chosen select elements */
  width: auto;
}


/*
 * Deform required flag
 */

span.req {
  font-weight: bold;
  color: #B94A48;
}



/*
 * Popover has a fixed width, hypenate at will
 */

.popover-content {
  word-break: break-all;
}


/*
**  =====================
**  SPECIFIC FORM STYLING
**  =====================
*/


/* --- Login --- */

#login-form {
  background-color: #fafafa;
}

#login-form .btn-small {
  padding: 11px 9px;
}

#login-form .btn-large {
  font-weight: bold;
}


/* ----- "Custom/Customer" report form ----- */

form#report-customer div[data-deform-name="users"]:before {
  /* form fields that should begin a new line */
  content: " ";
  display: block;
}


/*
 *
 * All tables that contain time entries or reports.
 *
 */

table.time-entries {
  color: black;
}

table.time-entries tbody th {
  border-right: 1px solid #e8e8cc;
}

table.time-entries tbody tr:nth-child(2n+1) th,
table.time-entries tbody tr:hover th {
  background-color: white;
}

@media only screen and (max-width: 1024px) {
    table.table-search td:nth-child(5),
    table.table-search th:nth-child(5),
    table.table-search td:nth-child(6),
    table.table-search th:nth-child(6)
    {display: none;}
}

@media only screen and (max-width: 640px) {
    table.table-search td:nth-child(1),
    table.table-search th:nth-child(1),
    table.table-search td:nth-child(4),
    table.table-search th:nth-child(4)
    {display: none;}
}

div#results dt{
  margin: 1.5em 0 0;
}
div#results dt.ticketclosed a{
  color: gray;
}

div#results .author{
  color: #090;  
}

div#results dd{
  font-size: 0.9em;
  margin: 0.2em 1em;
}


/* XXX unused? */

#scorebar {
   background-image: url(../images/bg_strength_gradient.jpg);
   background-repeat: no-repeat;
   background-position: 0 0;
   position:absolute;
   width: 100px;
   z-index: 0;
}

#scorebarBorder {
   background: #333;
   height: 16px;
   margin-bottom: 2px;
   width: 100px;
}

#score {
    color: #000;
    font-size: 85%;
    position:absolute;
    text-align: center;
    width: 100px;
    z-index: 10;
}


