.kanban a.editable-click {color: black}
.kanban .thumbnail {line-height:18px;}
.box_itm{
    margin-top: 10px;
    margin-right: 4px;
    margin-bottom: 4px;
    margin-left: 0px;
    padding-top: 0px;
    padding-right: 0px;
    padding-bottom: 0px;
    padding-left: 5px;
    color:#150517;
    background:#FFFFFF;
    font-size: 0.9em;
    border-left: 5px solid white;
}
.click{cursor:pointer;}
.clear{clear:both;}
.WIP{font-size:0.8em;text-align:left;margin:0.7em 0;color:#888888}
.kanban ul.task_pool { margin:0 0 9px 0;
                       min-height: 50px;
                       list-style: none;}
.ui-state-highlight {
  padding: 20px 0;
}
.ui-sortable-helper {
    transform: rotate(3deg);
    -moz-transform: rotate(3deg);
    -webkit-transform: rotate(3deg);
}

.project {
   display: block;
   padding: 3px;
   max-width: 140px;
   line-height: 0.9em;
   font-size: 0.9em;
   overflow: hidden;
   white-space: nowrap;
   text-overflow: ellipsis;
   background-color: red;
   color: white;
}

.priority {
   border-left: 5px solid red;
}
.ticket {
   display: block;
   font-weight: bold;
   width: 200px;
   padding: 10px 0;
}
.customer-request {
   display: block;
   margin-top: 5px;
   color: #AAA;
}
.number {
   display: block;
   font-size: 130%;
   color: white;
   background-color: #AAA;
   padding: 5px;
   text-align: right;
   min-width: 39px;
   border-bottom-left-radius: 4px;
}

.kanban div.oversized {
  background-color: red;
  color: white;
}

.kanban div.oversized a.editable-click{
  color: white;
}

.kanban .thumbnail{background: #E3E3E3}

body .modal {
   width: 750px;
   margin-left: -375px;
}

.preloader {
  text-align: center;
}

.preloader img{
display: inline-block;
*display:inline; /* ie7 fix */
float: none; /* this is the part that makes it work */
}

.online-users {
  padding: 0 10px;
}
.online-users img {
  margin: 0 2px;
  border: 1px solid #08C;
}
.users {
  text-align: right;
}

.users .collapse img {
  padding-top: 5px;
}

.nobacklog {
  top: 20px;
  position:relative;
}

.header_name input {
  margin-top: 5px;
  width: auto;
}

.header_name, .WIP {
  font-variant: small-caps;
  margin-left: 3px;
  font-weight: 600;
}
