.group-tickets > .control-label {
	font-size: 15px;
	font-weight: bold;
}

.group-tickets fieldset{
	border: 1px solid #ddd;
	-webkit-border-radius: 4px;
	-moz-border-radius: 4px;
	border-radius: 4px;
	padding: 1em;
	margin-bottom: 10px;
}

span.req {
	padding-left: 0.3em;
}

.help-block {
	margin-top: -9px;
	color: #aaa;
}