
.projects-board .thumbnail {
  padding: 10px;
  width: 280px;
  margin: .5em;
}

.projects-board .thumbnail .boarded-project {
  padding: 5px 0;
  min-height: 24px;
  clear: none;
}

.projects-board .thumbnail .btn-group {
  float: left;
}

.outstanding-tickets {
 /* float: left;
  padding-top: 6px;*/
  padding-left: 5px;
}

.outstanding-tickets .badge {
  margin-left: 1px;
  margin-right: 1px;
}

.outstanding-tickets-table {
  width: 100%;
}

.outstanding-priority-1 {
  background-color: #ffd0d0;
  font-weight: bold;
}

.outstanding-priority-2 {
  background-color: #ffe0e0;
  font-weight: bold;
}

.outstanding-priority-3,
.outstanding-priority-4,
.outstanding-priority-5,
.outstanding-priority-6 {
  background-color: white;
}


/** other projects **/
.projects-listing {
  margin-top: 20px;
}
.projects-listing div.projects-list {
  margin-left: 1.5%;
}

.row-fluid [class*="span"] {
  margin-left: 1.164102564%;
}

li .spark{
    margin-right: 20px;
}

.widget-head .customer {
  font-size: 11px;
}

.btn.btn-app.btn-mini {
  width: 50px;
  font-size: 15px;
  border-radius: 5px;
  padding-bottom: 7px;
  padding-top: 8px;
}

.btn.btn-app {
  display: inline-block;
  width: 100px;
  margin: 2px;
  position: relative;
  font-size: 18px;
  font-weight: normal;
  color: #FFF;
  text-align: center;
  text-shadow: 0 -1px -1px rgba(0,0,0,0.2)!important;
  border: 0;
  border-radius: 12px;
  padding: 12px 0 8px;}

.btn {
  display: inline-block;
  color: #FFF!important;
  text-shadow: 0 -1px 0 rgba(0,0,0,0.25)!important;
  background-image: none!important;
  border: 5px solid;
  border-radius: 0;
  box-shadow: none!important;
  -webkit-transition: all ease .15s;
  -moz-transition: all ease .15s;
  -o-transition: all ease .15s;
  transition: all ease .15s;
  cursor: pointer;
  vertical-align: middle;
  margin: 0;
  position: relative;
  padding: 0 12px 1px;
  line-height: 32px;
  font-size: 14px;
}
.btn-mini {
  padding: 0 5px;
  line-height: 22px;
  border-width: 2px;
  font-size: 12px;
}

.btn.btn-app.btn-mini>[class*=icon] {
  display: block;
  font-size: 20px;
  line-height: 20px;
  margin: 0;
}
